est_multi_poly3 <- function(S,yv=rep(1,ns),k1,k2,X=NULL,start=0,link=0,disc=0,difl=0,
                           multi=1:J,piv=NULL,Phi=NULL,gac=NULL,De=NULL,fort=FALSE,tol=10^-10,
                           disp=FALSE,output=FALSE,multi2){
                           	
# WARNING: WORKS ONLY WITH BINARY RESPONSES AND RASCH/2PL MODEL   
# THE FUNCTION USES AN ADDITIONAL TYPE OF ABILITY FOR CERTAIN ITEMS IN multi2
# INDEPENDENCE BETWEEN THE LATENT VARIABLES

#        [piv,Th,Bec,gac,fv,Phi,Pp,lk,np,aic,bic] = est_multi_poly(S,yv,k,start,link,disc,difl,multi,piv,Th,bec,gac)
#
# Fit Latent Class model and some restricted versions with k classes for ordinal (NA for missing data)
# 
# S    : matrix of available configurations
# X    : matrix of corresponding covariates affecting the ability
# yv   : frequencies of the available configurations
# k    : number of latent classes
# X    : matrix of covariates for the multinomial logit on the class weights
# start: type of startgine values (0 = deterministic, 1 = random)
# link : type of link (0 = LC, 1 = GRM, 2 = PCM)
# disc : discriminating indices (0 = constrained, 1 = free)
# difl : difficulty levels (0 = free, 1 = additive decomposition)
# lk   : maximum log-likelihood
# piv  : weights of the latent classes
# Phi  : conditional distributions given the latent classes
# np   : number of free parameters
# bic  : Bayesian information criterion
# Th,be,ga : parameters for the model 4 (Th=Psi if start==3)
# fv   : list of items constrained
# fort : T for using fortran code for covariates, F using R code only
# tol  : relative tolerance level for convergence
# disp : TRUE for displying the likelihood evolution step by step
# outputs : to return additional outputs (Phi,Pp,Piv)
# th2  : propensity to answer

# With k=1
	cat("*-------------------------------------------------------------------------------*\n")
	if(k1==1 & k2==1){
	  cat("fit only for LC model with no other imput\n")
	  link = 0; disc = 0; difl = 0; multi = 1:dim(S)[2]
	  X = NULL
	}
	if(max(S,na.rm=TRUE)==1 & difl!=0){
	  cat("with binary data put difl=0\n")
	  difl = 0		
	}
# Preliminaries
# check problems with standard errors
    cov = !is.null(X)
    if(cov) X = as.matrix(X)
    miss = any(is.na(S))
	ns = nrow(S); J = ncol(S)
    if(miss){
    	cat("Missing data in the dataset, units and items without responses are removed\n")
    	ind = which(apply(is.na(S),1,all))
    	if(length(ind)>0){
        	S = S[-ind,]; yv = yv[-ind]
        	if(!is.null(X)) X = as.matrix(X[-ind,])
        	ind = which(apply(is.na(S),2,all))
        	if(length(ind)>0){
        	    S = S[,-ind]
	            miss = any(is.na(S))
	        }
	    }
    }
    if(miss){R=1*(!is.na(S)); S[is.na(S)]=0}
	l = max(S)+1
	ns = nrow(S); J = ncol(S)
	n = sum(yv)
# checks about the covariates
    if(cov){
    	ncov = ncol(X)
    	out = aggr_data(X)
    	Xdis = out$data_dis; Xlabel = out$label; Xndis = max(out$label)
    	XXdis1 = array(0,c(k1,(k1-1)*(ncov+1),Xndis))
    	if(k1==2) II = as.matrix(c(0,1)) else {II = diag(k1); II = II[,-1]}
    	for(i in 1:Xndis) XXdis1[,,i] = II%x%t(c(1,Xdis[i,]))
    	XXdis2 = array(0,c(k2,(k2-1)*(ncov+1),Xndis))
    	if(k2==2) II = as.matrix(c(0,1)) else {II = diag(k2); II = II[,-1]}
    	for(i in 1:Xndis) XXdis2[,,i] = II%x%t(c(1,Xdis[i,]))
    }
# about models
	if(link==1) ltype = "g" else if(link==2) ltype = "l"
	if(link == 1 || link == 2){
		if(is.vector(multi)) rm = 1
		else rm = nrow(multi)
		De1 = matrix(0,J,rm)
		if(rm==1){
			De1 = 1
			fv = multi[1]
		}else{
			for(r in 1:rm){
				ind = multi[r,]
				ind = ind[ind>0]
				De1[ind,r] = 1      
			}
			fv = multi[,1]     # list of constrained items
		}
		fve = (fv-1)*(l-1)+1
		indga = 1:max(multi); indga = indga[-fv]
		indga21 = multi2[1]; indga22 = multi2[-1]
		indth = 1:(k1*rm)
		indth2 = k1*rm+(1:k2)
		if(difl==0){
			indbe = k1*rm+k2+(1:(J*(l-1)-rm-1))
			indbec = 1:(J*(l-1)); indbec = indbec[-c(fve,multi2[1])]
		}else{
			indbe = k*rm+k+(1:(J-rm+l-2));
			indbec = 1:J; indbec = indbec[-fv]
		}
# find non redundant response configurations for each item
		conf1 = NULL; conf2 = NULL
		for(j in 1:J) for(h in 0:l){
			if(any(S[,j]==h)){
				conf1 = c(conf1,j)
				conf2 = c(conf2,h)
			}
		}
		nconf = length(conf1)
# abililities for each item
		if(rm==1) abils=rep(1,J) else{
			abils = rep(0,J)
			for(h in 1:rm){
				ind = multi[h,]; ind = ind[ind>0]
				abils[ind] = h
			}
		}
# design matrix
        # additional k columns for ZZ
		if(difl==0){
			ZZth = array(0,c(l-1,k1*rm,J*k1*k2))
			ZZth2 = array(0,c(l-1,k2,J*k1*k2))
			ZZbe = array(0,c(l-1,(l-1)*J-rm-1,J*k1*k2))
			ZZ = array(0,c(l-1,k1*rm+k2+(l-1)*J-rm-1,J*k1*k2))
		}else{
			if(difl==1) ZZ = array(0,c(l-1,k*rm+J-rm+l-2,J*k))
		}
		cont = 0; refitem = matrix(0,J*k1*k2,1)       # reference item of that design matrix
		one1 = matrix(1,1,k1); one2 = matrix(1,1,k2)
		for(c1 in 1:k1){
			for(c2 in 1:k2){			
				u1 = matrix(0,1,k1); u1[c1] = 1
				for(j in 1:J){
					u2 = matrix(0,1,rm)
					if(any(multi==j)) u2[abils[j]] = 1
					v = matrix(0,1,J); v[j] = 1
					cont = cont+1
					if(difl==0){
						Te = v%x%diag(l-1)
						Te = matrix(Te[,-c(fve,multi2[1])],l-1,dim(Te)[2]-length(fve)-1)
					}else if(difl==1){
						Te = cbind(v%x%rep(1,l-1),diag(l-1))
						Te = Te[,-c(fv,J+1)]
					}
               		u12 = matrix(0,1,k2)
               		if(any(multi2==j)) u12[c2] = 1
					ZZth[,,cont] = rep(1,l-1)%*%(u1%x%u2)
					ZZth2[,,cont] = rep(1,l-1)%*%u12
					ZZbe[,,cont] = -Te                
					if(l==2){
						ZZ[,,cont] = c(ZZth[,,cont],ZZth2[,,cont],ZZbe[,,cont])
					}else{
						ZZ[,,cont] = cbind(ZZth[,,cont],ZZth2[,,cont],ZZbe[,,cont])					
					}
					refitem[cont] = j
				}
			}
		}
	    confe2 = rep(conf2,k1*k2)   # response configuration
		confe1 = conf1          # corresponding design matrix in ZZ
		for(c in 2:(k1*k2)) confe1 = c(confe1,max(confe1)+conf1)
	}
# When there is just 1 latent class
#   if k == 1,
#     piv = 1;
#     P = zeros(2,J);
#     for j in 1:J,
#       for jb = 0:1,
#         ind = which(S[,j]==jb);
#         P(jb+1,j) =  sum(yv(ind))/n;
#       end
#     end
#     Psi = P;
#     psi = ones(ns,1);
#     for j in 1:J,
#       psi = psi.*P(S[,j]+1,j);
#     end
#     lk = yv"*log(psi);
#     np = J;
#     aic = -2*lk+2*np;
#     bic = -2*lk+np*log(n);
#     bec = NULL; gac = NULL;
#     Pp = ones(ns,1);
#     Th = NULL;
#     return
#   end
	out = matr_glob(l); Co = out$Co; Ma = out$Ma
# Starting values
	if(start == 0){
		if(cov){
		    de1 = rep(0,(k1-1)*(ncov+1))
		    de2 = rep(0,(k2-1)*(ncov+1)); piv = NULL
	    }else{
			be = NULL; piv = rep(1,k1*k2)/(k1*k2)
		} # latent class probabilities
		if(k1==1 & k2==1) grid = 0 else grid = (1:(k1*k2))-(k1*k2+1)/2#seq(-k,k,2*k/(k-1))
		Phi = array(0,c(l,J,k1*k2)) # probability every response
		for(j in 1:J){
			dist = rep(0,l)
			for(y in 0:(l-1)) dist[y+1] = sum(yv[S[,j]==y])/sum(yv[R[,j]==1])
#			browser()
			eta = Co%*%log(Ma%*%dist)
			for(c in 1:(k1*k2)) Phi[,j,c] = inv_glob(eta+grid[c])$p
		}
	}
	if(start==1){
		if(cov){
			de1 = rnorm((k1-1)*(ncov+1))/(rep(1,k1-1)%x%c(1,apply(X,2,sd)))
			de2 = rnorm((k2-1)*(ncov+1))/(rep(1,k2-1)%x%c(1,apply(X,2,sd)))
			piv = NULL
		}else{
			piv = runif(k1*k2)
			piv = piv/sum(piv)
		}
#		Phi = array(runif(l*J*k),c(l,J,k))
#		for(c in 1:k) for(j in 1:J) Phi[,j,c] = Phi[,j,c]/sum(Phi[,j,c]);
		Phi = array(0,c(l,J,k1*k2)) # probability every response
		for(j in 1:J){
			dist = rep(0,l)
			for(y in 0:(l-1)) dist[y+1] = sum(yv[S[,j]==y])/sum(yv[R[,j]==1])
			eta = Co%*%log(Ma%*%dist)
			for(c in 1:(k1*k2)) Phi[,j,c] = inv_glob(eta+rnorm(1))$p
		}
	}
	if(start==2) de = as.vector(De)  
	if(link==0){
		ga = NULL
	}else{
		if (start==0 || start==1){
			ga = rep(1,max(multi)-rm); ga2 = rep(1,length(multi2)-1)
		}else{
			ga = gac[-fv]
		}
	}
# Compute log-likelihood
	Psi = matrix(1,ns,k1*k2) # probability observed response
	if(miss){
		for(j in 1:J) for(c in 1:(k1*k2)) Psi[,c] = Psi[,c]*(Phi[S[,j]+1,j,c]*R[,j]+(1-R[,j]))
	}else{
    	if(fort){
            o = .Fortran("lk_obs",J,as.integer(k1*k2),as.integer(ns),as.integer(S),as.integer(l),Phi,Psi=Psi)
            Psi = o$Psi
        }else{
    	    for(j in 1:J) for(c in 1:(k1*k2)) Psi[,c] = Psi[,c]*Phi[S[,j]+1,j,c]
        }            	
	}
	if(cov){
	    Piv = matrix(0,ns,k1*k2)
	    out = prob_multilogit(XXdis1,de1,Xlabel,fort=fort)
	    Pdis1 = out$Pdis; Piv1 = out$P
	    out = prob_multilogit(XXdis2,de2,Xlabel,fort=fort)
	    Pdis2 = out$Pdis; Piv2 = out$P
	    for(i in 1:ns) Piv[i,] = Piv1[i,]%x%Piv2[i,]
	}else Piv = rep(1,ns)%o%(piv1%x%piv2)
	if(k1==1 & k2==1) Pj=Psi else Pj = Psi*Piv
	pm = rowSums(Pj)
	lk = sum(yv*log(pm))
	if(link==1 || link==2){
		cat(c("Model with multdimensional structure\n"))
		names = NULL
		for(j in 1:rm){names = c(names,paste("Dimension",j))}
		multi_out = as.matrix(multi)
		if(rm == 1) multi_out = t(multi_out)
		rownames(multi_out) = names
		print(multi_out)
		print(multi2)
	}
	cat(c("Link of type =                 ",link,"\n"))
	cat(c("Discrimination index =         ",disc,"\n"))
	cat(c("Constraints on the difficulty =",difl,"\n"))
	cat(c("Type of initialization =       ",start,"\n"))
#	browser()
	cat("*-------------------------------------------------------------------------------*\n")		
	if(disp){
		if(disc==0 || length(ga)==0){
    		cat("------------|-------------|-------------|-------------|-------------|-------------|-------------|-------------|\n");
    		cat("  iteration | clas (k1k2) |    model    |      lk     |    lk-lko   |     dis     |   min(par)  |   max(par)  |\n");
    		cat("------------|-------------|-------------|-------------|-------------|-------------|-------------|-------------|\n");
		}else if(disc==1){
			cat("------------|-------------|-------------|-------------|-------------|-------------|-------------|-------------|-------------|-------------|\n");
			cat("  iteration | clas (k1k2) |    model    |    lk       |    lk-lko   |      dis    |   min(ga)   |   max(ga)   |   min(par)  |   max(par)  |\n");
			cat("------------|-------------|-------------|-------------|-------------|-------------|-------------|-------------|-------------|-------------|\n");

		}
		cat(sprintf("%11g",c(0,k1*10+k2,link,lk)),"\n",sep=" | ")
	}
 	it = 0; lko = lk-10^10; dis = 0; par = 0; dga = NULL
# Iterate until convergence
	while(((abs(lk-lko)/abs(lko)>tol) && it<10^4) || it<2){
#t0 = proc.time()
		it = it+1
		paro = par; gao = ga; pivo = piv; deo = de; lko = lk
# ---- E-step ----
		V = ((yv/pm)%o%rep(1,k1*k2))*Piv*Psi
		I1 = diag(k1)%x%matrix(1,k2,1); V1 = V%*%I1; sV1 = colSums(V1) 
		I2 = matrix(1,k1,1)%x%diag(k2); V2 = V%*%I2; sV2 = colSums(V2)
#print(proc.time()-t0)
# ---- M-step ----
		if(link==0){  # LC model
			if(miss){
				for(j in 1:J) for(y in 1:l){
					ind = (S[,j]==(y-1))
					for(c in 1:k) Phi[y,j,c] = sum(V[ind,c]*R[ind,j])/sum(V[,c]*R[,j])
				}
			}else{
				for(j in 1:J) for(y in 1:l){
					ind = (S[,j]==y-1)
					for(c in 1:k) Phi[y,j,c] = sum(V[ind,c])/sV[c]
				}
			}
		}else{          # other models
			w = matrix(0,nconf,k1*k2)
			if(miss){
				for(conf in 1:nconf){
					j = conf1[conf]; h = conf2[conf]
					ind = (S[,j]==h)
					if(sum(ind)==1) w[conf,] = V[ind,]*R[ind,j] else w[conf,] = colSums(V[ind,]*R[ind,j])
				}				
		    	YY = t(matrix(as.vector(w),2,J*k1*k2))
			}else{
				if(it>1 & l==2 & fort){
    	            o = .Fortran("matr_YY",as.integer(J),as.integer(k),as.integer(ns),as.integer(S),V,YY=matrix(0,J*k,2))
    	            YY = o$YY
    			}else{
     				for(conf in 1:nconf){
	    				j = conf1[conf]; h = conf2[conf]
		    			w[conf,] = colSums(V[S[,j]==h,])
			    	}
			    	YY = t(matrix(as.vector(w),2,J*k))
				}
			}
#print(proc.time()-t0)			
			if(disc==1){
				if(it>1 & rm<J){
					ZZga = array(0,c(l-1,max(multi)+length(multi2)+(l-1)*J-rm-1,J*k1*k2))
					count = 0
					for(c in 1:(k1*k2)) for(j in 1:J){
						count = count+1
						if(any(multi==j)) ZZga[,j,count] = ZZth[,,count]%*%par[indth]
						if(any(multi2==j)){
							ZZga[,max(multi)+which(multi2==j),count] = ZZth2[,,count]%*%par[indth2]
						}
						ind = max(multi)+length(multi2)+(1:((l-1)*J-rm-1))
						ZZga[,ind,count] = ZZbe[,,count]
					}
					dimz = dim(ZZga)
					dimz[2] = dimz[2]-length(fv)-1
					ZZga = array(ZZga[,-c(fv,max(multi)+1),],dimz)
#print(proc.time()-t0)			
					if(l==2){
#						browser()
                        out = est_multilogit(YY,t(ZZga[1,,]),fort=fort,be=c(ga,ga2,par[indbe]),Pdis=P)
                        ga = out$be
           			    if(rcond(out$Fi)<10^-15){
           			    	print("singular information matrix")
#           			    	browser()
           			    }
					}else{
	    				ga = est_multi_glob(confe2,ZZ1,ltype,confe1,as.vector(w),ga)$be
					}
#					ind = 1:(J-rm+length(multi2)-1)
#					ga[ind] = pmin(pmax(ga[ind],-100),100) # limit
					ind = max(multi)-rm+length(multi2)-1+(1:((l-1)*J-rm-1))
					par[indbe] = ga[ind]; ga = ga[-ind]
					ind = max(multi)-rm+(1:(length(multi2)-1))
					ga2 = ga[ind]; ga = ga[-ind]
#					browser()
				}
#print(proc.time()-t0)			
				gac = rep(0,J); gac[1:max(multi)] = 1; gac[indga] = ga
				gac2 = rep(0,j); gac2[indga21] = 1; gac2[indga22] = ga2
				cont = 0
				for(c in 1:(k1*k2)){
					for(j in 1:J){
						cont = cont+1
						if(l==2){
							ZZ[,,cont] = c(gac[j]*ZZth[,,cont],gac2[j]*ZZth2[,,cont],ZZbe[,,cont])
						}else{
							ZZ[,,cont] = cbind(gac[j]*ZZth[,,cont],gac2[j]*ZZth2[,,cont],ZZbe[,,cont])	
						}
					}
				}
			}
#			print(length(confe2))
#			print(confe2)
#			print(dim(ZZ))
#			print(ZZ)
#			print(length(confe1))
#			print(confe1)
#			print(dim(w))
#			print(w)
#			return
#t1 = proc.time()-t0
            if(it>1 & l==2){
               out = est_multilogit(YY,ZZ,fort=fort,be=par,Pdis=P)
		    	par = out$be; P = out$P
			    if(rcond(out$Fi)<10^-15){
			    	print("singular information matrix")
#			    	browser()
			    }
            }else{
    			if(it==1){
    				out = est_multi_glob(confe2,ZZ,ltype,confe1,as.vector(w))   # update par
#    			 	browser()
    			}
	    		else out = est_multi_glob(confe2,ZZ,ltype,confe1,as.vector(w),par)
		    	par = out$be; P = out$P
		    }		
			Phi = array(t(P),c(l,J,k1*k2))
    	}
# Update piv
#print(proc.time()-t0)
        if(cov){
           out = est_multilogit(V1,XXdis1,Xlabel,de1,Pdis1,fort=fort)
		   de1 = out$be; Pdis1 = out$Pdis; Piv1 = out$P
           out = est_multilogit(V2,XXdis2,Xlabel,de2,Pdis2,fort=fort)
		   de2 = out$be; Pdis2 = out$Pdis; Piv2 = out$P
		   for(i in 1:ns) Piv[i,] = Piv1[i,]%x%Piv2[i,]
        }else{
            piv1 = sV1/n; piv2 = sV2/n
            piv = piv1%x%piv2
            Piv = rep(1,ns)%o%piv
       } 
#print(proc.time()-t0)
# Compute log-likelihood
		Psi = matrix(1,ns,k1*k2)
		if(miss){
			for(j in 1:J) for(c in 1:(k1*k2))	Psi[,c] = Psi[,c]*(Phi[S[,j]+1,j,c]*R[,j]+(1-R[,j]))	
		}else{
			if(fort){
                o = .Fortran("lk_obs",J,as.integer(k1*k2),as.integer(ns),as.integer(S),as.integer(l),Phi,Psi=Psi)
                Psi = o$Psi
            }else{
    			for(j in 1:J) for(c in 1:(k1*k2))	Psi[,c] = Psi[,c]*Phi[S[,j]+1,j,c]
    		}            	
		}
		if(k1==1 & k2==1) Pj=Psi else Pj = Psi*Piv
        pm = rowSums(Pj)
	    lk = sum(yv*log(pm))
		dis = max(c(abs(par-paro),abs(ga-gao),abs(piv-pivo)))
		if(disp){
			if(it/10==floor(it/10)){
				if(disc==0 || length(ga)==0) cat(sprintf("%11g",c(it,10*k1+k2,link,lk,lk-lko,dis,min(par),max(par))),"\n",sep=" | ") else{
				if(disc==1) cat(sprintf("%11g",c(it,10*k1+k2,link,lk,lk-lko,dis,min(ga),max(ga),min(par),max(par))),"\n",sep=" | ")
				}
#t2 = proc.time()-t0
#print(c(t1[1],t2[1]))
			}
		}
	}
	if(disp){
		if(it/10>floor(it/10)){
			if(disc==0 || length(ga)==0){
				cat(sprintf("%11g",c(it,10*k1+k2,link,lk,lk-lko,dis,min(par),max(par))),"\n",sep=" | ")
		    		cat("------------|-------------|-------------|-------------|-------------|-------------|-------------|-------------|\n")
			}else if(disc==1){
				cat(sprintf("%11g",c(it,10*k1+k2,link,lk,lk-lko,dis,min(ga),max(ga),min(par),max(par))),"\n",sep=" | ")
				cat("------------|-------------|-------------|-------------|-------------|-------------|-------------|-------------|-------------|-------------|\n")
			}
		}
	}
# Compute number of parameters  
	if(link == 0){
	  np = k*J*(l-1)
      if(cov) np = np+(k1+k2-2)*(ncov+1) else np = np+k1+k2-2
    }else if(link==1 || link==2){
        np = k1*rm+k2+disc*(max(multi)-rm+length(multi2)-1)
        if(cov) np = np+(k1+k2-2)*(ncov+1) else np = np+k1+k2-2
		if(difl==0) np = np+(l-1)*J-rm-1 else if(difl==1) np = np+J-rm+l-2
	}
# extract parameter estimates  
  aic = -2*lk+2*np;
  bic = -2*lk+np*log(n);
  if(link==0){
    Th = NULL; th2 = NULL; Bec = NULL; gac = NULL; gac2 = NULL; fv = NULL
  }
  else if(link==1 || link==2){
    th = par[indth]; th2 = par[indth2]; be = par[indbe]
    if(difl==0){
      bec = rep(0,J*(l-1)); bec[indbec] = be
      Bec = t(matrix(bec,l-1,J))
	  ind = J-rm+(1:(length(multi2)-1))
      gac = rep(0,J); gac[1:max(multi)] = 1; gac[indga] = ga
	  gac2 = rep(0,j); gac2[indga21] = 1; gac2[indga22] = ga2
    }
    else{
      bec1 = rep(0,J); bec1[indbec] = be[1:(J-rm)]
      bec2 = rep(0,l-1); bec2[2:(l-1)] = be[J-rm+(1:(l-2))]
      Bec = list(bec1=bec1,bec2 = bec2)
    }
    gac = rep(0,J); gac[1:max(multi)] = 1; gac[indga] = ga
    Th = matrix(th,rm,k1)
    names2 = NULL
    for(c in 1:k1){names2 = c(names2,paste("Class",c))}
    rownames(Th) = names
    colnames(Th) = names2
  }
  Pp = ((1./pm)%o%rep(1,k1*k2))*Piv*Psi
  if(cov){
      De1 = matrix(de1,ncov+1,k1-1)
      De2 = matrix(de2,ncov+1,k2-1)
      piv1 = colMeans(Piv1)
      piv2 = colMeans(Piv2)
  }else{
  	De1 = NULL
  	De2 = NULL
  }
  if(output){
    out = list(piv1=piv1,piv2=piv2,Th=Th,th2=th2,Bec=Bec,gac=gac,gac2=gac2,fv=fv,Phi=Phi,De1=De1,De2=De2,
               Piv1=Piv1,Piv2=Piv2,Pp=Pp,lk=lk,np=np,aic=aic,bic=bic)
  }else{
    out = list(piv1=piv1,piv2=piv2,Th=Th,th2=th2,Bec=Bec,gac=gac,gac2=gac2,fv=fv,De1=De1,De2=De2,lk=lk,np=np,aic=aic,bic=bic)
  }
}