# *** load package ***
require(MultiLCIRT)
source("est_multi_poly3.R")
tol = 10^-9
fort = FALSE # also possible to use fortran with option fort=TRUE (to speed up the code)

# *** load simulated data and prepare in the proper format ***
load("data_www.RData")
S[S>1] = NA
SR = cbind(S,!is.na(S))
m = 20 # number of items

# *** fit proposed model  ***

# deterministic starting (default)
multi = cbind(matrix(1:m,2,m/2,byrow=TRUE),matrix(1:m,2,m/2,byrow=TRUE)+m)
multi2 = (m+1):(2*m)
out_rasch = est_multi_poly3(SR,k1=3,k2=3,X=X,link=1,disc=0,multi=multi,multi2=multi2,tol=tol,fort=fort,disp=T)   #Rasch
out_2PL = est_multi_poly3(SR,k1=3,k2=3,X=X,link=1,disc=1,multi=multi,multi2=multi2,tol=tol,fort=fort,disp=T)   #2PL

# random starting (option start=1)
out_rasch1 = est_multi_poly3(SR,k1=3,k2=3,X=X,link=1,disc=0,multi=multi,multi2=multi2,tol=tol,fort=fort,disp=T,start=1)   #Rasch
out_2PL1 = est_multi_poly3(SR,k1=3,k2=3,X=X,link=1,disc=1,multi=multi,multi2=multi2,tol=tol,fort=fort,disp=T,start=1)   #2PL

# *** extract output with ordered classes ***
ind1 = order(out_2PL$Th[1,])  # regression coefficients for the covariates (ability)
De1 = cbind(0,out_2PL$De1) 
De1 = De1[,ind1]
De1 = De1-De1[,1]%o%rep(1,3)
De1 = De1[,-1]	
ind2 = order(out_2PL$th2)     # regression coefficients for the covariates (propensity to respond)
De2 = cbind(0,out_2PL$De2)
De2 = De2[,ind2]
De2 = De2-De2[,1]%o%rep(1,3)
De2 = De2[,-1]	
Th = out_2PL$Th[,ind1]        # support points (ability)
th2 = out_2PL$th2[ind2]       # support points (tendency to respond)
piv1 = out_2PL$piv1           # mass probabilities (ability)
piv2 = out_2PL$piv2           # mass probabilities (tendency to respond)
out.st = standard.matrix(t(Th),piv1)   # standardize distributions
Cor = out.st$Cor
mu1 = out.st$mu; si1 = out.st$si
Th = (Th-mu1%o%rep(1,3))/(si1%o%rep(1,3))
mu2 = th2%*%piv2; si2 = sqrt((th2^2)%*%piv2-mu2^2)
th2 = (th2-mu2)/si2
Bec = out_2PL$Bec
gac = out_2PL$gac
gac2 = out_2PL$gac2
for(d in 1:2){
	Bec[multi[d,]] = Bec[multi[d,]]-gac[multi[d,]]*mu1[d]
	gac[multi[d,]] = gac[multi[d,]]*si1[d]
}
Bec[multi2] = Bec[multi2]-gac2[multi2]*mu2
Bec = as.vector(Bec)
gac2 = gac2*si2
save.image("example_www.RData")

# *** Bootstrap for confidence intervals ***
n = nrow(X)
B = 199
ksel = 3
De1_boot = array(0,c(dim(De1),B))
De2_boot = array(0,c(dim(De2),B))
Bec_boot = matrix(0,length(Bec)[1],B)
Gac_boot = matrix(0,length(gac),B)
Gac2_boot = matrix(0,length(gac2),B)
Cor_boot = array(0,c(2,2,B))
for(b in 1:B){
	cat(paste("bootstrap sample n.",b),"\n")
	ind = sample(n,n,replace=T)
	SRb = SR[ind,]
	while(any(colSums(SRb[,(m+1):(2*m)])==n)){
    	ind = sample(n,n,replace=T)
	    SRb = SR[ind,]
	}
	# use option fort = TRUE to speed up the code
    modb = est_multi_poly3(SRb,k1=ksel,k2=ksel,X=X[ind,],link=1,disc=1,multi=multi,multi2=multi2,tol=tol,fort=fort,disp=T)
	ind1 = order(modb$Th[1,])
	De1b = cbind(0,modb$De1)
	De1b = De1b[,ind1]
	De1b = De1b-De1b[,1]%o%rep(1,ksel)
	Thb = modb$Th[,ind1]
	ind2 = order(modb$th2) 
	De2b = cbind(0,modb$De2)
	De2b = De2b[,ind2]
	De2b = De2b-De2b[,1]%o%rep(1,ksel)
	th2b = modb$th2[ind2]
	piv1b = modb$piv1[ind1]
	piv2b = modb$piv2[ind2]
	out.st = standard.matrix(t(Thb),piv1b)
	Corb = out.st$Cor
	mu1b = out.st$mu; si1b = out.st$si
	Thb = (Thb-mu1b%o%rep(1,3))/(si1b%o%rep(1,3))
	mu2b = th2b%*%piv2b; si2b = sqrt((th2b^2)%*%piv2b-mu2b^2)
	th2b = (th2b-mu2b)/si2b
	Becb = modb$Bec
	gacb = modb$gac
	gac2b = modb$gac2
	for(d in 1:2){
		Becb[multi[d,]] = Becb[multi[d,]]-gacb[multi[d,]]*mu1b[d]
	  	gacb[multi[d,]] = gacb[multi[d,]]*si1b[d]
  	}
	Becb[multi2] = Becb[multi2]-gac2b[multi2]*mu2b
	gac2b = gac2b*si2b
	De1_boot[,,b] = De1b[,-1]	
	De2_boot[,,b] = De2b[,-1]	
	Bec_boot[,b] = Becb
	Gac_boot[,b] = gacb
	Gac2_boot[,b] = gac2b
	Cor_boot[,,b] = Corb
	if(b>1){
	  	mDe1 = apply(De1_boot[,,1:b],c(1,2),median)
		seDe1 = apply(De1_boot[,,1:b],c(1,2),sd)
		mDe2 = apply(De2_boot[,,1:b],c(1,2),median)
		seDe2 = apply(De2_boot[,,1:b],c(1,2),sd)
		mBec = apply(Bec_boot[,1:b],1,median)
		seBec = apply(Bec_boot[,1:b],1,sd)
		mGac = apply(Gac_boot[,1:b],1,median)
		seGac = apply(Gac_boot[,1:b],1,sd)
		mGac2 = apply(Gac2_boot[,1:b],1,median)
		seGac2 = apply(Gac2_boot[,1:b],1,sd)
		mCor = apply(Cor_boot[,,1:b],c(1,2),median)
		seCor = apply(Cor_boot[,,1:b],c(1,2),sd)
   }
   save.image("example_www.RData")
}